# Class: pe_puppetdb::pe
#
# This class is a wrapper for puppetdb and pe_postgresql modules.
# It provides a simple way to get a PE puppetdb instance up and running
# within Puppet Enterprise.  It will install and configure all necessary packages,
# including the database PostgreSQL server and instance.
#
# In addition to this class, you'll need to configure your puppet master to use
# puppetdb.  You can use the `puppetdb::master::config` class to accomplish this.
#
# Parameters:
#   ['listen_address']     - The address that the web server should bind to
#                            for HTTP requests.  (defaults to `localhost`.
#                            '0.0.0.0' = all)
#   ['listen_port']        - The port on which the puppetdb web server should
#                            accept HTTP requests (defaults to 8080).
#   ['open_listen_port']   - If true, open the http listen port on the firewall.
#                            (defaults to false).
#   ['ssl_listen_address'] - The address that the web server should bind to
#                            for HTTPS requests.  (defaults to `$::clientcert`.)
#                            Set to '0.0.0.0' to listen on all addresses.
#   ['ssl_listen_port']    - The port on which the puppetdb web server should
#                            accept HTTPS requests (defaults to 8081).
#   ['open_ssl_listen_port'] - If true, open the ssl listen port on the firewall.
#                            (defaults to true).
#   ['database_host']      - The host the database server runs on.
#                            (defaults to `localhost`; ignored for `embedded` db)
#   ['database_port']      - The port that the database server listens on.
#                            (defaults to `5432`; ignored for `embedded` db)
#   ['database_username']  - The name of the database user to connect as.
#                            (defaults to `puppetdb`; ignored for `embedded` db)
#   ['database_password']  - The password for the database user.
#                            (defaults to `puppetdb`; ignored for `embedded` db)
#   ['database_name']      - The name of the database instance to connect to.
#                            (defaults to `puppetdb`; ignored for `embedded` db)
#   ['tablespace_name']    - The name of puppetdb tablespace.
#   ['tablespace_location']- The location of puppetdb tablespace.
#   ['manage_database']    - Whether to manage the Postgres database. Defaults
#                            to true, can be set to false if the database is
#                            remote and already exists.
#   ['node_ttl']           - The length of time a node can go without receiving
#                            any new data before it's automatically deactivated.
#                            (defaults to '0', which disables auto-deactivation)
#                            This option is supported in PuppetDB >= 1.1.0.
#   ['node_purge_ttl']     - The length of time a node can be deactivated before
#                            it's deleted from the database.
#                            (defaults to '0', which disables purging)
#                            This option is supported in PuppetDB >= 1.2.0.
#   ['report_ttl']         - The length of time reports should be stored before
#                            being deleted.
#                            (defaults to '7d', which is a 7-day period)
#                            This option is supported in PuppetDB >= 1.1.0.
#   ['puppetdb_package']   - The puppetdb package name in the package manager
#   ['puppetdb_version']   - The version of the `puppetdb` package that should
#                            be installed.  You may specify an explicit version
#                            number, 'present', or 'latest'.  (defaults to
#                            'present')
#   ['puppetdb_service']   - The name of the puppetdb service.
#   ['confdir']            - The puppetdb configuration directory; defaults to
#                            `/etc/puppetdb/conf.d`.
#   ['java_args']          - Java VM options used for overriding default Java VM
#                            options specified in PuppetDB package.
#                            e.g. { '-Xmx' => '512m', '-Xms' => '256m' }
#   ['pe_postgresql_config_hash'] - The hash of parameters and their values
#                            of the pe_postgresql::config class.
#   ['postgresql_conf_params'] - The hash of parameters and their values to be
#                            managed in postgres.conf - the PostgreSQL
#                            configurtion file.
#
# Actions:
# - Creates and manages a puppetdb server and its database server/instance.
#
# Requires:
# - `puppetlabs/puppetlabs-pe_postgresql`
#
# Sample Usage:
#   include pe_puppetdb::pe
#
class pe_puppetdb::pe(
  $listen_address            = $pe_puppetdb::params::listen_address,
  $listen_port               = $pe_puppetdb::params::listen_port,
  $open_listen_port          = $pe_puppetdb::params::open_listen_port,
  $ssl_listen_address        = $pe_puppetdb::params::ssl_listen_address,
  $ssl_listen_port           = $pe_puppetdb::params::ssl_listen_port,
  $open_ssl_listen_port      = $pe_puppetdb::params::open_ssl_listen_port,
  $database_host             = $pe_puppetdb::pe::params::database_host,
  $database_port             = $pe_puppetdb::pe::params::database_port,
  $database_username         = $pe_puppetdb::pe::params::database_username,
  $database_password         = $pe_puppetdb::pe::params::database_password,
  $database_name             = $pe_puppetdb::pe::params::database_name,
  $tablespace_name           = $pe_puppetdb::pe::params::tablespace_name,
  $tablespace_location       = $pe_puppetdb::pe::params::tablespace_location,
  $manage_database           = true,
  $node_ttl                  = $pe_puppetdb::pe::params::node_ttl,
  $node_purge_ttl            = $pe_puppetdb::params::node_purge_ttl,
  $report_ttl                = $pe_puppetdb::params::report_ttl,
  $puppetdb_package          = $pe_puppetdb::pe::params::puppetdb_package,
  $puppetdb_version          = $pe_puppetdb::params::puppetdb_version,
  $puppetdb_service          = $pe_puppetdb::pe::params::puppetdb_service,
  $confdir                   = $pe_puppetdb::pe::params::confdir,
  $postgres_listen_addresses = undef,
  $java_args                 = pe_puppetdb_hiera_hash_pick(
    # check this for backward compatibility
    'pe_puppetdb::java_args'
  ),
  $pe_postgresql_config_hash = {},
  $postgresql_conf_params    = pe_puppetdb_hiera_hash_pick(
    # check these for backward compatibility
    'pe_puppetdb::pe::database::database_config_hash',
    'pe_puppetdb::database::database_config_hash'
  )
) inherits pe_puppetdb::pe::params {
  class { 'pe_puppetdb::server':
    listen_address          => $listen_address,
    listen_port             => $listen_port,
    open_listen_port        => $open_listen_port,
    ssl_listen_address      => $ssl_listen_address,
    ssl_listen_port         => $ssl_listen_port,
    open_ssl_listen_port    => $open_ssl_listen_port,
    database                => $pe_puppetdb::pe::params::database,
    database_host           => $database_host,
    database_port           => $database_port,
    database_username       => $database_username,
    database_password       => $database_password,
    database_name           => $database_name,
    node_ttl                => $node_ttl,
    node_purge_ttl          => $node_purge_ttl,
    report_ttl              => $report_ttl,
    puppetdb_package        => $puppetdb_package,
    puppetdb_version        => $puppetdb_version,
    puppetdb_service        => $puppetdb_service,
    confdir                 => $confdir,
    java_args               => merge(
      $pe_puppetdb::pe::params::default_jvm_args,
      $java_args
    ),
  }

  if ($manage_database) {
    # get the pg server up and running
    $minimal_pe_postgresql_config_hash = {
      'ip_mask_allow_all_users' => '0.0.0.0/0',
       # If this isn't set explicitly, it should be whatever PuppetDB is expecting
       # to connect to postgres on.
      'listen_addresses'        => $postgres_listen_addresses ? {
        default => $postgres_listen_addresses,
        undef   => $database_host
      },
      'manage_redhat_firewall'  => false,
    }

    class { 'pe_postgresql::pe':
      config_hash => merge(
        $minimal_pe_postgresql_config_hash,
        $pe_postgresql_config_hash
      ),
      before      => Class['pe_puppetdb::server'],
    }

    # PostgreSQL configuration

    # 'memorysize' fact in bytes (can be used in postgresql_conf_params)
    $memorysize_in_bytes = to_bytes($memorysize)

    # reserved memory for PuppetDB (can be used in postgresql_conf_params)
    # '0 + x' is used for conversion to integer
    $reserved_non_postgresql_memory_in_bytes = 0 + pick(
      pe_puppetdb_hiera_pick(
        'pe_puppetdb::pe::reserved_non_postgresql_memory_in_bytes',
        # check this for backward compatibility
        'pe_puppetdb::pe::database::reserved_non_postgresql_memory_in_bytes'
      ),
      $pe_puppetdb::pe::params::default_reserved_non_postgresql_memory_in_bytes
    )

    $merged_postgresql_conf_params = merge(
      $pe_puppetdb::pe::params::default_postgresql_conf_params,
      $postgresql_conf_params
    )

    if !empty($merged_postgresql_conf_params) {
      create_resources(
        'pe_postgresql_conf',
        pe_puppetdb_create_postgresql_conf_resource_hash(
          $merged_postgresql_conf_params,
          {
            ensure  => present,
            target  => $pe_postgresql::params::postgresql_conf_path,
            require => Class['::pe_postgresql::initdb'],
            before  => Class['pe_puppetdb::server'],
            notify  => Service['postgresqld'],
          }
        )
      )
    }

    # create tablespace for puppetdb database
    pe_postgresql::tablespace { $tablespace_name:
      location => $tablespace_location,
      require  => [Class['::pe_postgresql::server'], Service['postgresqld']],
      before   => Class['pe_puppetdb::server'],
    }

    # create the puppetdb database
    pe_postgresql::db { $database_name:
      tablespace => $tablespace_name,
      user       => $database_username,
      password   => $database_password,
      grant      => 'all',
      require    => Pe_postgresql::Tablespace[$tablespace_name],
      before     => Class['pe_puppetdb::server'],
    }
  }
}
